from rest_framework import serializers


class PUTSignatureSerializer(serializers.Serializer):
    file_name = serializers.CharField()
    mime_type = serializers.CharField()


class GETSignatureSerializer(serializers.Serializer):
    url = serializers.CharField()



from setuptools import find_packages, setup

setup(
    name="djassr",
    version=__import__('djassr').__version__,
    author="Alexandre Varas",
    author_email="alej0varas@gmail.com",
    license='GNU Library or Lesser General Public License (LGPL)',
    description="Django AWS S3 Signed Requests API with Django Rest Framework",
    url='https://github.com/alej0varas/djassr',
    packages=('djassr', ),
    install_requires=['s3sign==0.2.0'],
    tests_require=['s3sign==0.2.0', 'django==1.9.7', 'djangorestframework==3.3.3'],
    test_suite='runtests',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django :: 1.9',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
    ],
)



import urllib
import uuid

from rest_framework import generics
from rest_framework.response import Response

import s3sign

from . import serializers


DEFAULT_VALID = 60  # seconds


class BaseGetSignature(generics.GenericAPIView):
    def post(self, request):
        args = self._get_args(request)
        signer = self.signer()
        data = signer.get_signed_url(*args)
        return Response(data)

    def get_valid(self, request):
        return DEFAULT_VALID


class BaseGetPUTSigneature(BaseGetSignature):
    serializer_class = serializers.PUTSignatureSerializer

    def _get_args(self, request):
        file_name = self.get_object_name(request)
        mime_type = request.data.get('mime_type')
        valid = self.get_valid(request)
        return file_name, valid, mime_type

    def get_object_name(self, request):
        file_name = request.data.get('file_name')
        extension = file_name.split('.')[-1]
        file_name = str(uuid.uuid4()) + '.' + extension
        object_name = urllib.parse.quote_plus(file_name)
        return object_name


class GetPUTSignature(BaseGetPUTSigneature):
    signer = s3sign.S3PUTSigner


class GetPUTPublicSignature(BaseGetPUTSigneature):
    signer = s3sign.S3PUTPublicSigner


class GetGETSignature(BaseGetSignature):

    serializer_class = serializers.GETSignatureSerializer

    signer = s3sign.S3GETSigner

    def _get_args(self, request):
        valid = self.get_valid(request)
        return request.data.get('object_name'), valid
